package com.fjhx.account.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.account.entity.cost.dto.CostControlDto;
import com.fjhx.account.entity.cost.po.CostControl;
import com.fjhx.account.service.cost.CostControlService;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 费控流程
 *
 * @author y
 * @since 2024-03-19
 */
@Component
public class CostControlFlow extends FlowDelegate {

    @Autowired
    private CostControlService costControlService;
    @Autowired
    private CodingRuleService codingRuleService;

    @Override
    public String getFlowKey() {
        return "cost_control_flow";
    }

    /**
     * 发起流程
     */
    @Override
    public Long start(Long flowId, JSONObject submitData) {
        CostControlDto costControlDto = submitData.toJavaObject(CostControlDto.class);
        costControlDto.setFlowId(flowId);
        costControlDto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        costControlDto.setCode(codingRuleService.createCode(CodingRuleEnum.COST_CONTROL.getKey(), null));

        //保存数据
        costControlService.addOrEdit(costControlDto);

        //回传数据给流程引擎
        submitData.put("code", costControlDto.getCode());
        submitData.put("amount", costControlDto.getAmount());

        return costControlDto.getId();
    }

    /**
     * 结束流程
     */
    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        costControlService.update(q -> q
                .eq(CostControl::getId, businessId)
                .set(CostControl::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );
    }

    /**
     * 重新发起
     */
    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        CostControlDto costControlDto = submitData.toJavaObject(CostControlDto.class);
        costControlService.addOrEdit(costControlDto);
    }

    /**
     * 驳回
     */
    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        costControlService.update(q -> q
                .eq(CostControl::getId, businessId)
                .set(CostControl::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        costControlService.cancellation(businessId);
    }

}
