package com.fjhx.account.service.account.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.account.entity.account.dto.AccountSubjectsDto;
import com.fjhx.account.entity.account.dto.AccountSubjectsSelectDto;
import com.fjhx.account.entity.account.po.AccountSubjects;
import com.fjhx.account.entity.account.vo.AccountSubjectsVo;
import com.fjhx.account.mapper.account.AccountSubjectsMapper;
import com.fjhx.account.service.account.AccountSubjectsService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 记账科目 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-18
 */
@Service
public class AccountSubjectsServiceImpl extends ServiceImpl<AccountSubjectsMapper, AccountSubjects> implements AccountSubjectsService {

    @Override
    public List<AccountSubjectsVo> getList(AccountSubjectsSelectDto dto) {
        IWrapper<AccountSubjects> wrapper = getWrapper();

        wrapper.keyword(dto.getKeyword(), new SqlField("asu.subjects_name"), new SqlField("calculate_item_name"));

        wrapper.orderByDesc("asu", AccountSubjects::getId);
        List<AccountSubjectsVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<AccountSubjectsVo> getPage(AccountSubjectsSelectDto dto) {
        IWrapper<AccountSubjects> wrapper = getWrapper();

        wrapper.keyword(dto.getKeyword(), new SqlField("asu.subjects_name"), new SqlField("calculate_item_name"));

        wrapper.orderByDesc("asu", AccountSubjects::getId);
        Page<AccountSubjectsVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public AccountSubjectsVo detail(Long id) {
        AccountSubjects AccountSubjects = this.getById(id);
        AccountSubjectsVo result = BeanUtil.toBean(AccountSubjects, AccountSubjectsVo.class);
        return result;
    }

    @Override
    public void add(AccountSubjectsDto accountSubjectsDto) {
        Long parentId = accountSubjectsDto.getParentId();
        if (parentId == null || parentId == 0L) {
            accountSubjectsDto.setParentId(0L);
            accountSubjectsDto.setParentIdSet(null);
        } else {
            // 查询父级分类
            AccountSubjects parentClassify = getById(parentId);
            if (parentClassify == null) {
                throw new ServiceException("没有找到父级分类");
            }

            // 赋值父级id集合
            String parentIdSet = parentClassify.getParentIdSet();
            accountSubjectsDto.setParentIdSet((ObjectUtil.isEmpty(parentIdSet) ? "" : parentIdSet + ",") + parentId);
        }

        this.save(accountSubjectsDto);
    }

    @Override
    public void edit(AccountSubjectsDto accountSubjectsDto) {
        this.updateById(accountSubjectsDto);
    }

    @Override
    public void delete(Long id) {
        this.remove(q -> q
                .eq(AccountSubjects::getId, id)
                .or().apply("FIND_IN_SET({0},parent_id_set)", id)
        );
    }

}
