package com.fjhx.account.service.cost.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.account.entity.cost.dto.CostControlDetailDto;
import com.fjhx.account.entity.cost.dto.CostControlDto;
import com.fjhx.account.entity.cost.dto.CostControlSelectDto;
import com.fjhx.account.entity.cost.po.CostControl;
import com.fjhx.account.entity.cost.po.CostControlDetail;
import com.fjhx.account.entity.cost.vo.CostControlDetailVo;
import com.fjhx.account.entity.cost.vo.CostControlVo;
import com.fjhx.account.mapper.cost.CostControlMapper;
import com.fjhx.account.service.cost.CostControlDetailService;
import com.fjhx.account.service.cost.CostControlService;
import com.fjhx.common.controller.Label;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.logistics.LogisticsCompanyInfoService;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.entity.flow.po.FlowExample;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 费控 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-18
 */
@Service
public class CostControlServiceImpl extends ServiceImpl<CostControlMapper, CostControl> implements CostControlService {
    @Autowired
    private LogisticsCompanyInfoService logisticsCompanyInfoService;
    @Autowired
    private CostControlDetailService costControlDetailService;
    @Autowired
    private FlowExampleService flowExampleService;

    @Override
    public Page<CostControlVo> getPage(CostControlSelectDto dto) {
        IWrapper<CostControl> wrapper = getWrapper();
        wrapper.orderByDesc("cc", CostControl::getId);
        Page<CostControlVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public CostControlVo detail(Long id) {
        CostControl CostControl = this.getById(id);
        CostControlVo result = BeanUtil.toBean(CostControl, CostControlVo.class);

        //赋值详情
        setInfo(Arrays.asList(result));

        //赋值明细列表
        List<CostControlDetailVo> costControlDetailVos = costControlDetailService.getList(IWrapper.getWrapper().eq(CostControlDetail::getCostControlId, result.getId()));
        result.setCostControlDetailList(costControlDetailVos);

        return result;
    }

    private void setInfo(List<CostControlVo> costControlVos) {
        //赋值物流公司信息
        logisticsCompanyInfoService.attributeAssign(costControlVos, CostControlVo::getLogisticsCompanyId, (detail, logistics) -> {
            detail.setLogisticsName(logistics.getName());
            detail.setLogisticsTaxPoints(logistics.getTaxPoints());
            detail.setLogisticsAccountName(logistics.getAccountName());
            detail.setLogisticsAccountBank(logistics.getAccountBank());
            detail.setLogisticsAccountNumber(logistics.getAccountNumber());
        });
        //赋值申请人信息
        UserUtil.assignmentNickName(costControlVos, CostControlVo::getApplyUserId, CostControlVo::setApplyUserName);
    }

    @Override
    @DSTransactional
    public void addOrEdit(CostControlDto costControlDto) {
        this.saveOrUpdate(costControlDto);

        //保存明细信息
        List<CostControlDetailDto> costControlDetailList = costControlDto.getCostControlDetailList();
        if (ObjectUtil.isEmpty(costControlDetailList)) {
            costControlDetailList = new ArrayList<>();
        }

        //删除无效数据
        List<Long> idList = costControlDetailList.stream().map(BaseIdPo::getId).filter(Objects::nonNull).collect(Collectors.toList());
        costControlDetailService.remove(q -> q.eq(CostControlDetail::getCostControlId, costControlDto.getId()).notIn(ObjectUtil.isNotEmpty(idList), BaseIdPo::getId, idList));
        for (CostControlDetailDto costControlDetailDto : costControlDetailList) {
            costControlDetailDto.setCostControlId(costControlDto.getId());
            costControlDetailService.saveOrUpdate(costControlDetailDto);
            //保存非发票附件
            ObsFileUtil.editFile(costControlDetailDto.getFileList(), costControlDetailDto.getId(), 0);
            //保存发票附件
            ObsFileUtil.editFile(costControlDetailDto.getFileList(), costControlDetailDto.getId(), 10);
        }
    }

    @Override
    public void edit(CostControlDto costControlDto) {
        this.updateById(costControlDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public JSONObject getFieldInfo() {
        List<JSONObject> infoField = getField(CostControlVo.class);
        List<JSONObject> detailField = getField(CostControlDetailVo.class);

        JSONObject json = new JSONObject();
        json.put("infoField", infoField);
        json.put("detailField", detailField);


        return json;
    }

    @Override
    public void cancellation(Long id) {
        this.update(q -> q
                .eq(CostControl::getId, id)
                .set(CostControl::getStatus, FlowStatusEnum1.CANCELLATION.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //销毁审批中的流程
        CostControl byId = getById(id);
        flowExampleService.update(q -> q
                .eq(FlowExample::getId, byId.getFlowId())
                .in(FlowExample::getStatus, FlowStatusEnum.READY_START.getKey(), FlowStatusEnum.IN_PROGRESS.getKey())
                .set(FlowExample::getStatus, FlowStatusEnum.CANCELLATION.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );
    }

    private List<JSONObject> getField(Class cl) {
        //获取所有字段
        List<Field> fieldList = new ArrayList<>();
        Collections.addAll(fieldList, cl.getDeclaredFields());
        Class<? super CostControlVo> superclass = cl.getSuperclass();
        if (ObjectUtil.isNotEmpty(superclass)) {
            Collections.addAll(fieldList, superclass.getDeclaredFields());
        }

        //遍历处理
        List<JSONObject> arr = new ArrayList<>();
        for (Field field : fieldList) {
            Label labelAnnotation = field.getAnnotation(Label.class);
            if (labelAnnotation != null) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("key", field.getName());
                jsonObject.put("val", labelAnnotation.value());
                arr.add(jsonObject);
            }
        }
        return arr;
    }

}
