package com.fjhx.purchase.service.subscribe.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.item.entity.product.vo.ProductInfoVo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.purchase.entity.purchase.po.Purchase;
import com.fjhx.purchase.entity.purchase.po.PurchaseDetail;
import com.fjhx.purchase.entity.subscribe.dto.DecisionAidDto;
import com.fjhx.purchase.entity.subscribe.dto.SubscribeDetailSelectDto;
import com.fjhx.purchase.entity.subscribe.dto.SubscribeDto;
import com.fjhx.purchase.entity.subscribe.dto.SubscribeSelectDto;
import com.fjhx.purchase.entity.subscribe.po.Subscribe;
import com.fjhx.purchase.entity.subscribe.po.SubscribeDetail;
import com.fjhx.purchase.entity.subscribe.vo.DecisionAidVo;
import com.fjhx.purchase.entity.subscribe.vo.SubscribeDetailVo;
import com.fjhx.purchase.entity.subscribe.vo.SubscribeVo;
import com.fjhx.purchase.mapper.subscribe.SubscribeMapper;
import com.fjhx.purchase.service.purchase.PurchaseDetailService;
import com.fjhx.purchase.service.purchase.PurchaseService;
import com.fjhx.purchase.service.subscribe.SubscribeDetailService;
import com.fjhx.purchase.service.subscribe.SubscribeService;
import com.fjhx.supply.entity.supplier.po.SupplierInfo;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.fjhx.wms.entity.warehouse.po.Warehouse;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.utils.UserUtil;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 申购单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-03
 */
//@DS(SourceConstant.PURCHASE)
@Service
public class SubscribeServiceImpl extends ServiceImpl<SubscribeMapper, Subscribe> implements SubscribeService {
    private final SubscribeDetailService subscribeDetailService;
    private final ProductInfoService productInfoService;
    private final ISysDeptService deptService;
    private final CodingRuleService codingRuleService;
    private final PurchaseService purchaseService;
    private final PurchaseDetailService purchaseDetailService;
    private final WarehouseService warehouseService;
    private final SupplierInfoService supplierInfoService;

    @Autowired
    public SubscribeServiceImpl(SubscribeDetailService subscribeDetailService, ProductInfoService productInfoService, ISysDeptService deptService, CodingRuleService codingRuleService, PurchaseService purchaseService, PurchaseDetailService purchaseDetailService, WarehouseService warehouseService, SupplierInfoService supplierInfoService) {
        this.subscribeDetailService = subscribeDetailService;
        this.productInfoService = productInfoService;
        this.deptService = deptService;
        this.codingRuleService = codingRuleService;
        this.purchaseService = purchaseService;
        this.purchaseDetailService = purchaseDetailService;
        this.warehouseService = warehouseService;
        this.supplierInfoService = supplierInfoService;
    }


    @Override
    public Page<SubscribeVo> getPage(SubscribeSelectDto dto) {
        IWrapper<Subscribe> wrapper = getWrapper();

        //权限过滤：申购
        wrapper.in("s", Subscribe::getCompanyId, SecurityUtils.getCompanyIds());
        wrapper.eq("s", Subscribe::getCompanyId, dto.getCompanyId());


        wrapper.orderByDesc("s", Subscribe::getId);
        Page<SubscribeVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @Override
    public SubscribeVo detail(Long id) {
        if (ObjectUtils.isEmpty(id)) {
            throw new ServiceException("参数异常");
        }
        Subscribe subscribe = this.getById(id);
        if (ObjectUtils.isEmpty(subscribe)) {
            throw new ServiceException("数据不存在");
        }
        SubscribeVo subscribeVo = BeanUtil.copyProperties(subscribe, SubscribeVo.class);

        //处理申购产品
        List<SubscribeDetail> subscribeDetailList = subscribeDetailService.list(q -> q.eq(SubscribeDetail::getSubscribeId, id));
        List<SubscribeDetailVo> subscribeDetailVos = BeanUtil.copyToList(subscribeDetailList, SubscribeDetailVo.class);
        if (CollectionUtils.isNotEmpty(subscribeDetailList)) {
            //根据申购businessId取出所有产品
            List<Long> productIds = subscribeDetailList.stream().map(SubscribeDetail::getProductId).collect(Collectors.toList());
            List<ProductInfoVo> productInfoVos = productInfoService.getListByProductIds(productIds);
            Map<Long, List<ProductInfoVo>> productMap = productInfoVos.stream().collect(Collectors.groupingBy(ProductInfoVo::getId));
            for (SubscribeDetailVo s : subscribeDetailVos) {
                if (MapUtils.isNotEmpty(productMap)) {
                    ProductInfoVo p = productMap.get(s.getProductId()).get(0);
                    s.setProductCategory(p.getClassifyName());
                    s.setProductCode(p.getCustomCode());
                    s.setProductUnit(p.getUnit());
                    s.setProductType(p.getType());
                    s.setProductName(p.getName());
                    s.setProductDefinition(p.getDefinition());

                    s.setProductLength(p.getLength());
                    s.setProductWidth(p.getWidth());
                    s.setProductHeight(p.getHeight());

                    s.setProductColor(p.getColor());
                    s.setProductNetWeight(p.getNetWeight());
                    s.setProductFrontalTexture(p.getFrontalTexture());
                    s.setProductReverseTexture(p.getReverseTexture());
                }
            }
        }
        subscribeVo.setSubscribeDetailList(subscribeDetailVos);

        //赋值仓库信息
        Warehouse warehouse = warehouseService.getById(subscribeVo.getPutWarehouseId());
        if (ObjectUtil.isNotEmpty(warehouse)) {
            subscribeVo.setPutWarehouseName(warehouse.getName());
        }
        //赋值供应商信息
        SupplierInfo supplierInfo = supplierInfoService.getById(subscribeVo.getSupplierId());
        if (ObjectUtil.isNotEmpty(supplierInfo)) {
            subscribeVo.setSupplierName(supplierInfo.getName());
            subscribeVo.setSupplierContactPerson(supplierInfo.getContactPerson());
            subscribeVo.setSupplierContactNumber(supplierInfo.getContactNumber());
            subscribeVo.setSupplierPrivIncludingTax(supplierInfo.getPrivIncludingTax());
            subscribeVo.setSupplierPrivTaxPoints(supplierInfo.getPrivTaxPoints());
        }
        //赋值部门信息
        SysDept sysDept = deptService.getById(subscribeVo.getDeptId());
        if (ObjectUtil.isNotEmpty(sysDept)) {
            subscribeVo.setDeptName(sysDept.getDeptName());
        }
        //赋值公司信息
        SysDept company = deptService.getById(subscribeVo.getDeptId());
        if (ObjectUtil.isNotEmpty(company)) {
            subscribeVo.setCompanyName(company.getDeptName());
        }

        //赋值申购人
        UserUtil.assignmentNickName(Arrays.asList(subscribeVo), Subscribe::getSubcribeUserId, SubscribeVo::setSubcribeName);

        return subscribeVo;
    }

    @DSTransactional
    @Override
    public void add(SubscribeDto subscribe) {
        subscribe.setSubcribeStatus(FlowStatusEnum1.DRAFT.getKey());
        saveOrEdit(subscribe);
    }

    /**
     * 编辑
     *
     * @param subscribeDto
     */
    @DSTransactional
    @Override
    public void edit(SubscribeDto subscribeDto) {
        subscribeDto.setSubcribeStatus(FlowStatusEnum1.DRAFT.getKey());
        saveOrEdit(subscribeDto);
    }

    @Override
    public void saveOrEdit(SubscribeDto subscribe) {
        //赋值归属公司
        if (ObjectUtils.isEmpty(subscribe.getCompanyId())) {
            subscribe.setCompanyId(SecurityUtils.getCompanyId());
        }
        this.saveOrUpdate(subscribe);

        List<SubscribeDetail> subscribeDetails = subscribe.getSubscribeDetailList();
        if (ObjectUtils.isEmpty(subscribeDetails)) {
            subscribeDetails = new ArrayList<>();
        }
        subscribeDetails.forEach(item -> {
            item.setSubscribeId(subscribe.getId());
            item.setDataType(0);
            item.setCompanyId(subscribe.getCompanyId());
        });
        subscribeDetailService.editLinked(subscribeDetails, SubscribeDetail::getSubscribeId, subscribe.getId());

        //代他人提交附件
        ObsFileUtil.saveFile(subscribe.getSubstituteFileList(), subscribe.getId(), 10);
        //申请用途附件
        ObsFileUtil.saveFile(subscribe.getPurposeFileList(), subscribe.getId(), 15);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public List<SysDept> getDepts(SubscribeDetailSelectDto subscribeDetailDto) {
        List<Long> deptIds = new ArrayList<>();
        List<SubscribeDetail> list;
        String status = subscribeDetailDto.getStatus();
        if (ObjectUtils.isEmpty(status)) {
            list = subscribeDetailService.list();
        } else {
            String[] split = status.split(",");
            list = subscribeDetailService.list(q -> q.in(SubscribeDetail::getStatus, split));
        }
        if (ObjectUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        if (ObjectUtils.isEmpty(deptIds)) {
            return new ArrayList<>();
        }
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        List<SysDept> sysDepts = deptService.listByIds(deptIds);
        DynamicDataSourceContextHolder.poll();
        return sysDepts;
    }

    @Override
    public DecisionAidVo decisionAid(DecisionAidDto dto) {
        DecisionAidVo decisionAidVo = new DecisionAidVo();

        Long supplyId = dto.getSupplyId();
        if (supplyId != null) {
            List<Purchase> purchaseList = purchaseService.list(q -> q.eq(Purchase::getSupplyId, supplyId)
                    .orderByDesc(BaseIdPo::getId).last("limit 3"));
            decisionAidVo.setPurchaseList(purchaseList);
        }

        // 产品价格
        List<Long> productIdList = dto.getProductIdList();
        if (ObjectUtil.isNotEmpty(productIdList)) {

            Map<Long, List<PurchaseDetail>> map = purchaseDetailService.mapKGroup(PurchaseDetail::getBussinessId, q -> q
                    .select(PurchaseDetail::getBussinessId, PurchaseDetail::getPrice)
                    .in(PurchaseDetail::getBussinessId, productIdList)
                    .orderByDesc(BaseIdPo::getId)
            );

            List<DecisionAidVo.ProductPrice> productPriceList = new ArrayList<>();

            for (Long productId : productIdList) {
                List<PurchaseDetail> contractProductList = map.get(productId);
                DecisionAidVo.ProductPrice productPrice = new DecisionAidVo.ProductPrice();
                productPrice.setId(productId);
                if (ObjectUtil.isNotEmpty(contractProductList)) {
                    productPrice.setLastPrice(contractProductList.get(0).getPrice());
                    productPrice.setMaxPrice(contractProductList.stream().map(PurchaseDetail::getPrice).max(BigDecimal::compareTo).get());
                    productPrice.setMinPrice(contractProductList.stream().map(PurchaseDetail::getPrice).min(BigDecimal::compareTo).get());
                } else {
                    productPrice.setLastPrice(BigDecimal.ZERO);
                    productPrice.setMaxPrice(BigDecimal.ZERO);
                    productPrice.setMinPrice(BigDecimal.ZERO);
                }
                productPriceList.add(productPrice);

            }

            productInfoService.attributeAssign(productPriceList, DecisionAidVo.ProductPrice::getId, (item, product) -> {
                item.setName(product.getName());
            });

            decisionAidVo.setProductPriceList(productPriceList);
        } else {
            decisionAidVo.setProductPriceList(Collections.emptyList());
        }

        return decisionAidVo;
    }

}
