package com.fjhx.sale.flow;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.utils.Assert;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.sale.entity.contract.dto.ContractOutboundInfoDto;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.po.ContractOutboundInfo;
import com.fjhx.sale.entity.contract.vo.ContractOutboundInfoVo;
import com.fjhx.sale.service.contract.ContractOutboundInfoService;
import com.fjhx.sale.service.contract.ContractService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class SalesOutboundApprovedFlow extends FlowDelegate {
    @Autowired
    private ContractService contractService;
    @Autowired
    private ContractOutboundInfoService contractOutboundInfoService;

    @Override
    public String getFlowKey() {
        return "sales_outbound_approved_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        ContractOutboundInfoDto dto = submitData.toJavaObject(ContractOutboundInfoDto.class);
        dto.setId(null);
        dto.setFlowId(flowId);
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());
        contractService.createOutboundInfo(dto);

        Contract contract = contractService.getById(dto.getContractId());
        Assert.notEmpty(contract, "查询不到合同信息");
        //将code赋值给流程引擎
        submitData.put("code", contract.getCode());
        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        contractOutboundInfoService.update(q -> q
                .eq(ContractOutboundInfo::getId, businessId)
                .set(ContractOutboundInfo::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );
        ContractOutboundInfoVo detail = contractOutboundInfoService.detail(businessId);
        ContractOutboundInfoDto dto = BeanUtil.copyProperties(detail, ContractOutboundInfoDto.class);
        //特批无需校验金额
        contractService.saleOutboundComm(dto);
    }
}
