package com.fjhx.sale.mapper.contract;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.account.entity.account.vo.AccountRunningWaterVo;
import com.fjhx.customer.entity.customer.dto.CustomerDto;
import com.fjhx.item.entity.product.bo.ProductAnalysisBo;
import com.fjhx.sale.entity.contract.dto.ContractDto;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.vo.ContractBudgetVo;
import com.fjhx.sale.entity.contract.vo.ContractProductVo;
import com.fjhx.sale.entity.contract.vo.ContractVo;
import com.fjhx.sale.entity.sale.vo.SaleQuotationVo;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;


/**
 * <p>
 * 外销合同表 Mapper 接口
 * </p>
 *
 * @author
 * @since 2023-04-17
 */
@Mapper
public interface ContractMapper extends BaseMapper<Contract> {

    /**
     * 外销合同表分页
     */
    Page<ContractVo> getPage(@Param("page") Page<Object> page, @Param("ew") IWrapper<Contract> wrapper);

    /**
     * 外销合同表分页
     */
    Map<String, Object> getHeadStatistic(@Param("ew") IWrapper<Contract> wrapper);

    /**
     * 利润预算表
     */
    Page<ContractBudgetVo> getProfitBudgetPage(@Param("page") Page<Object> page, @Param("ew") IWrapper<Contract> wrapper);

    /**
     * 利润预算表头部统计
     *
     * @param wrapper
     * @return
     */
    List<ContractBudgetVo> getProfitBudgetHeadStatistic(@Param("ew") IWrapper<Contract> wrapper);

    /**
     * 利润结算表
     */
    Page<ContractBudgetVo> getProfitClearingPage(@Param("page") Page<Object> page, @Param("ew") IWrapper<Contract> wrapper);

    /**
     * 根据客户ID查询未包装的订单
     *
     * @param customerId
     * @return
     */
    List<ContractVo> getNoPackContractByCustomerId(@Param("customerId") String customerId);

    /**
     * 查询销售额（合同总金额）
     *
     * @param id (买方公司ID)
     */
    ContractVo getSalesTotal(Long id);

    /**
     * 查询销售额明细
     *
     * @param id
     * @return
     */
    Page<ContractVo> getSalesTotalList(@Param("page") Page<Object> page, Long id);

    /**
     * 查询指定客户的每月合同总额
     *
     * @param dto
     */
    @TenantIgnore
    List<ContractVo> getAmount(CustomerDto dto);

    /**
     * 查询成交单统计（合同）
     *
     * @param query
     * @return
     */
    List<SaleQuotationVo> contractStatistics(@Param("ew") QueryWrapper<Object> query);

    /**
     * 国家销售统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> countrySalesStatistics(@Param("ew") QueryWrapper<Object> query);

    /**
     * 客户销售统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> customSalesStatistics(@Param("ew") QueryWrapper<Object> query);

    /**
     * 业务员销售统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> salesmanSalesStatistics(@Param("ew") QueryWrapper<Object> query);

    /**
     * 产品销售统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> productSalesStatistics(@Param("ew") QueryWrapper<Object> query);

    List<ContractDto> salesVolumeStatistics(@Param("ew") QueryWrapper<Object> query);

    /**
     * 根据合同id获取合同的资金流水信息
     */
    List<AccountRunningWaterVo> getAccountRunningWaterByContractId(@Param("contractId") Long contractId);


    Page<ContractVo> getContractAndSamplePagez(@Param("page") Page<Object> page, @Param("ew") IWrapper<Contract> wrapper);

    void updateContract(Contract contract);

    /**
     * 外销跟单分页
     *
     * @param page
     * @param wrapper
     * @return
     */
    Page<ContractVo> getExportCopyList(@Param("page") Page<Object> page, @Param("ew") IWrapper<Contract> wrapper, @Param("documentaryType") Integer documentaryType);

    /**
     * 获取合同产品分析
     */
    List<ProductAnalysisBo> getContractProductInfo(@Param("ew") IWrapper<Object> wrapper);

    /**
     * 获取采购产品分析
     */
    List<ProductAnalysisBo> getPurchaseProductInfo(@Param("ew") IWrapper<Object> wrapper);

    List<ContractProductVo> getProductPriceInfo(@Param("ew") IWrapper<Object> wrapper);

    /**
     * 获取合同生产订单数
     */
    long getProductionOrderCount(@Param("ew") IWrapper<Object> wrapper);

    /**
     * 删除合同相关的生产订单
     */
    void removeProductionOrderByContractId(Long contractId);

    /**
     * 删除合同相关的生产任务
     */
    void removeProductionOrderDetailByContractId(Long contractId);

    /**
     * 删除合同相关的生产备料
     */
    void removeMaterialPreparationoByContractId(Long contractId);

    /**
     * 修改生产订单交期确认状态
     */
    void setProductionOrderConfirmStatus(Long id, Integer confirmStatus);

    void setProdOrderProdStatusByContractId(Long contractId, Integer produceStatus);


    void setProdTaskProdStatusByContractId(Long contractId, Integer produceStatus);

    ContractVo detail(Long id);

}
