package com.fjhx.sale.service.claim.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.entity.claim.ClaimContract;
import com.fjhx.common.entity.claim.dto.ClaimContractDto;
import com.fjhx.common.entity.claim.dto.ClaimContractSelectDto;
import com.fjhx.common.entity.claim.vo.ClaimContractVo;
import com.fjhx.sale.mapper.claim.ClaimContractMapper;
import com.fjhx.sale.service.claim.ClaimContractService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 到账认领-合同关联表 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-04-18
 */
@Service
public class ClaimContractServiceImpl extends ServiceImpl<ClaimContractMapper, ClaimContract> implements ClaimContractService {

    @Override
    public Page<ClaimContractVo> getPage(ClaimContractSelectDto dto) {
        IWrapper<ClaimContract> wrapper = getWrapper();
        wrapper.orderByDesc("cc", ClaimContract::getId);
        Page<ClaimContractVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public List<ClaimContractVo> getList(@Param("ew") IWrapper<ClaimContract> wrapper) {
        List<ClaimContractVo> page = this.baseMapper.getList(wrapper);
        return page;
    }

    @Override
    public ClaimContractVo detail(Long id) {
        ClaimContract ClaimContract = this.getById(id);
        ClaimContractVo result = BeanUtil.toBean(ClaimContract, ClaimContractVo.class);
        return result;
    }

    @Override
    public void add(ClaimContractDto claimContractDto) {
        this.save(claimContractDto);
    }

    @Override
    public void edit(ClaimContractDto claimContractDto) {
        this.updateById(claimContractDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 通过合同ID集合查询列表
     * @param contractIds
     * @return
     */
    @Override
    public List<ClaimContract> getListByContractIds(List<Long> contractIds) {
        IWrapper<ClaimContract> wrapper = getWrapper();
        wrapper.in("t1.contract_id", contractIds);
        return baseMapper.getListByContractIds(wrapper);
    }

    /**
     * 根据合同号查询流水ID
     * @param code
     * @return
     */
    @Override
    public List<Long> getAccountWaterIdsByContractCode(String code) {
        return baseMapper.getAccountWaterIdsByContractCode(code);
    }

}
