package com.fjhx.sale.service.contract.impl;

import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.sale.entity.contract.dto.ContractOutboundRecordsSelectDto;
import com.fjhx.sale.entity.contract.po.ContractOutboundRecords;
import com.fjhx.sale.entity.contract.vo.ContractOutboundRecordsVo;
import com.fjhx.sale.mapper.contract.ContractOutboundRecordsMapper;
import com.fjhx.sale.service.contract.ContractOutboundRecordsService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 销售出库详情 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-13
 */
@Service
public class ContractOutboundRecordsServiceImpl extends ServiceImpl<ContractOutboundRecordsMapper, ContractOutboundRecords> implements ContractOutboundRecordsService {
    @Autowired
    private ProductInfoService productInfoService;

    @Override
    public Page<ContractOutboundRecordsVo> getPage(ContractOutboundRecordsSelectDto dto) {
        IWrapper<ContractOutboundRecords> wrapper = getWrapper();
        wrapper.eq("cor.contract_id", dto.getContractId());
        wrapper.in("coi.status", 10, 30);
        Page<ContractOutboundRecordsVo> page = baseMapper.getPage(dto.getPage(), wrapper);
        List<ContractOutboundRecordsVo> records = page.getRecords();
        if (ObjectUtils.isEmpty(records)) {
            return page;
        }
        //赋值产品信息
        productInfoService.attributeAssign(records, ContractOutboundRecordsVo::getProductId, (item, productInfo) -> {
            item.setProductCode(productInfo.getCustomCode());
            item.setProductName(productInfo.getName());
            item.setProductLength(productInfo.getLength());
            item.setProductWidth(productInfo.getWidth());
            item.setProductHeight(productInfo.getHeight());
            item.setProductColor(productInfo.getColor());

            item.setProductUnit(productInfo.getUnit());
            item.setProductNetWeight(productInfo.getNetWeight());
            item.setProductReverseTexture(productInfo.getReverseTexture());
            item.setProductFrontalTexture(productInfo.getFrontalTexture());
        });
        return page;
    }

    @Override
    public List<ContractOutboundRecordsVo> getList(IWrapper wrapper) {
        return baseMapper.getList(wrapper);
    }
}
